import 'dart:convert';

import 'package:app_settings/app_settings.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/CustomCountdownTimer.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/NoDataFoundWidget.dart';
import 'package:flyyfluttersdk/model/FlyyTournament.dart';
import 'package:flyyfluttersdk/model/FlyyTournamentData.dart';
import 'package:flyyfluttersdk/views/FlyyGameDetailsPage.dart';

import 'FlyyTournamentHistoryPage.dart';

///Stateless widget
class FlyyGameZonePage extends StatelessWidget {
  FlyyTournament flyyTournament;
  NoDataFoundWidget noDataFoundWidget;

  @override
  Widget build(BuildContext context) {
    noDataFoundWidget = new NoDataFoundWidget();
    return Scaffold(
      appBar: AppBar(
        bottomOpacity: 0.0,
        elevation: 0.0,
        titleSpacing: 0.0,
        leading: IconButton(
          icon: Icon(Icons.arrow_back, color: Colors.white),
          onPressed: () {
            Navigator.of(context, rootNavigator: false).pop();
          },
        ),
        title: Text(
          "Game Zone",
          style: TextStyle(fontSize: 18),
        ),
        actions: <Widget>[
          Container(
            margin: const EdgeInsets.only(right: 10),
            child: IconButton(
              icon: Image.network(
                "https://ghc.theflyy.com/assets/fs_assets/history.png",
                height: 35,
                width: 35,
              ),
              onPressed: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(
                      builder: (context) => FlyyTournamentHistoryPage(
                            prizeType: null,
                          )),
                );
              },
            ),
          )
        ],
        backgroundColor: Flyy.TOOLBAR_COLOR,
      ),
      body: makeNetworkCall(context),
    );
  }

  ///fetch data
  Widget makeNetworkCall(BuildContext context) {
    return FutureBuilder<bool>(
      future: FlyyUtility.checkIfInterentPresent(),
      builder: (BuildContext context, AsyncSnapshot response) {
        if (response != null && response.data != null && response.data) {
          // Internet Present Case
          return FutureBuilder<FlyyTournament>(
            future: fetchTournaments(),
            builder: (BuildContext context, AsyncSnapshot response) {
              if (response.connectionState == ConnectionState.waiting) {
                return Center(child: CircularProgressIndicator());
              } else {
                if (response.hasData && response.data != null) {
                  flyyTournament = response.data;

                  if (flyyTournament.success) {
                    return FlyyGameZoneStateFulWidget(
                      flyyTournament: flyyTournament,
                    );
                  } else {
                    return noDataFoundWidget.showNoDataLayout(
                        flyyTournament.message);
                  }
                } else if (response.hasError) {
                  print(response.error);
                }
              }
              return Center(child: CircularProgressIndicator());
            },
          );
        } else {
          // No-Internet Case
          return noInternetLayout(context);
        }
      },
    );
  }

  ///fetch tournaments api call
  Future<FlyyTournament> fetchTournaments() async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("fetch_tournaments?partner=" +
        Flyy.PARTNER_TOKEN +
        "&user_name=" +
        " " +
        "&contact_number=" +
        Flyy.EXT_USER_ID);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      flyyTournament = FlyyTournament.fromJson(responseJson);

      return flyyTournament;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }

  ///no internet layout
  Widget noInternetLayout(BuildContext context) {
    return Container(
      margin: const EdgeInsets.all(20),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Image.network(
            "https://ghc.theflyy.com/assets/fs_assets/ic_no_internet.png",
            height: 80,
            width: 80,
          ),
          Padding(
            padding: const EdgeInsets.only(top: 10, bottom: 10),
            child: Text(
              "Couldn't connect to the internet. Please check your network.",
              textAlign: TextAlign.center,
              style: TextStyle(
                  fontSize: 15, color: FlyyUtility.parseColor('#575555')),
            ),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              GestureDetector(
                onTap: () {
                  makeNetworkCall(context);
                },
                child: Row(
                  children: <Widget>[
                    Icon(
                      Icons.refresh,
                      size: 30,
                      color: FlyyUtility.parseColor('#717171'),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Tap To Retry',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
              GestureDetector(
                onTap: () {
                  AppSettings.openWIFISettings();
                },
                child: Row(
                  children: <Widget>[
                    Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_settings.png",
                      height: 30,
                      width: 30,
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Go To Settings',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
            ],
          )
        ],
      ),
    );
  }
}

///Stateful widget
class FlyyGameZoneStateFulWidget extends StatefulWidget {
  FlyyTournament flyyTournament;

  FlyyGameZoneStateFulWidget({Key key, @required this.flyyTournament})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyGameZoneState();
  }
}

///State class
class FlyyGameZoneState extends State<FlyyGameZoneStateFulWidget> {
  FlyyTournament flyyTournament;
  List<FlyyTournamentData> tournamentDataList = new List();
  FlyyUtility flyyUtility;

  ///Call fetch info and load UI when response is success.
  @override
  Widget build(BuildContext context) {
    flyyUtility = new FlyyUtility();
    flyyTournament = widget.flyyTournament;
    tournamentDataList.addAll(flyyTournament.tournamentsList);
    if (tournamentDataList.length <= 0) {
      NoDataFoundWidget noDataFoundWidget = new NoDataFoundWidget();
      return noDataFoundWidget.showNoDataLayout("No tournaments live");
    }
    else
      return displayGamesCardUI();
  }

  Widget displayGamesCardUI() {
    return Container(
      color: FlyyCommonColors.BG_GREY,
      child: ListView.builder(
        itemBuilder: setGamesListView,
        itemCount: tournamentDataList.length,
        scrollDirection: Axis.vertical,
      ),
    );
  }

  Widget setGamesListView(BuildContext context, int index) {
    FlyyTournamentData flyyTournamentData =
        flyyTournament.tournamentsList[index];
    return Container(
      child: Card(
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(5),
        ),
        child: Column(
          children: <Widget>[
            ///banner and timer layout
            Stack(
              alignment: Alignment.topRight,
              children: <Widget>[
                Padding(
                  padding: const EdgeInsets.all(10),
                  child: Image.network(
                    flyyTournamentData.gameBanner,
                    height: 150,
                    fit: BoxFit.fill,
                    width: double.maxFinite,
                  ),
                ),
                Container(
                  height: 50,
                  width: 150,
                  padding: const EdgeInsets.all(5),
                  margin: const EdgeInsets.only(top: 20, right: 5),
                  color: FlyyUtility.parseColor("#d02828"),
                  child: Column(
                    children: <Widget>[
                      Padding(
                        padding: const EdgeInsets.only(bottom: 5),
                        child: Text("LIVE",
                            style:
                                TextStyle(fontSize: 16, color: Colors.white)),
                      ),
                      Row(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: <Widget>[
                          Text("Ends:",
                              style:
                                  TextStyle(fontSize: 12, color: Colors.white)),
                          setCountDownTimerText(flyyTournamentData.endsAt,
                              flyyTournamentData.isLive),
                        ],
                      ),
                    ],
                  ),
                ),
              ],
            ),

            ///title, prize and play button layout
            Padding(
              padding: const EdgeInsets.only(right: 10, left: 10, bottom: 10),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text(
                        flyyTournamentData.title,
                        textAlign: TextAlign.left,
                        style: TextStyle(
                            fontSize: 14,
                            fontWeight: FontWeight.bold,
                            color: FlyyUtility.parseColor("#585957")),
                      ),
                      Padding(
                        padding: const EdgeInsets.only(top: 5),
                        child: Row(
                          children: <Widget>[
                            Text(
                              "Total Prize: ",
                              style: TextStyle(
                                  fontSize: 14,
                                  color: FlyyUtility.parseColor("#585957")),
                            ),
                            Image.network(
                              flyyTournamentData.prizeIconUrl,
                              height: 20,
                              width: 20,
                            ),
                            Padding(
                              padding: const EdgeInsets.only(left: 5),
                              child: Text(
                                flyyUtility.setRewardText(
                                    flyyTournamentData.totalPrize,
                                    flyyTournamentData.prizeType),
                                style: TextStyle(
                                    fontSize: 14,
                                    color: FlyyUtility.parseColor("#585957")),
                              ),
                            ),
                          ],
                        ),
                      ),
                    ],
                  ),

                  ///play game button
                  GestureDetector(
                    onTap: () {
                      if (flyyTournamentData.isLive != null &&
                          flyyTournamentData.isLive) {
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                              builder: (context) => FlyyGameDetailsPage(
                                    flyyTournament: flyyTournament,
                                    flyyTournamentData: flyyTournamentData,
                                  )),
                        );
                      } else {
                        FlyyUtility.showToast(context, "Tournament has ended..",
                            ToastGravity.BOTTOM);
                      }
                    },
                    child: Container(
                      height: 35,
                      width: 100,
                      child: Card(
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(15),
                        ),
                        color: Flyy.TOOLBAR_COLOR,
                        child: Center(
                          child: Text(
                            'Play',
                            style: new TextStyle(
                                fontSize: 14, color: Colors.white),
                          ),
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget setCountDownTimerText(String endsAt, bool isLive) {
    DateTime endDate = DateTime.parse(endsAt);

    return CustomCountdownTimer(
      onEnd: () {
        fetchTournaments();
      },
      endTime: endDate.millisecondsSinceEpoch,
      showDays: true,
      daysSymbol: "d:",
      hoursSymbol: "h:",
      minSymbol: "m:",
      secSymbol: "s",
      textStyle: TextStyle(fontSize: 12, color: Colors.white),
    );
  }

  Future<FlyyTournament> fetchTournaments() async {
    APIClient apiClient = new APIClient();
    final response = await apiClient.getClient("fetch_tournaments?partner=" +
        Flyy.PARTNER_TOKEN +
        "&user_name=" +
        " " +
        "&contact_number=" +
        Flyy.EXT_USER_ID);
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      FlyyTournament flyyTournament = FlyyTournament.fromJson(responseJson);
      if (flyyTournament.tournamentsList != null &&
          flyyTournament.tournamentsList.length > 0)
        tournamentDataList.addAll(flyyTournament.tournamentsList);

      return flyyTournament;
    } else {
      throw Exception('Failed to load data ' + response.toString());
    }
  }
}
