import 'dart:async';
import 'dart:io';

import 'package:android_intent/android_intent.dart';
import 'package:countdown/countdown.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/model/FlyyPlayGame.dart';
import 'package:webview_flutter/webview_flutter.dart';

WebViewController controllerGlobal;

class FlyyPlayGamesPage extends StatelessWidget {
  FlyyPlayGame flyyPlayGame;

  FlyyPlayGamesPage({Key key, @required this.flyyPlayGame}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomPadding: false,
      body: FlyyPlayGamesStatefulWidget(
        flyyPlayGame: flyyPlayGame,
      ),
    );
  }
}

class FlyyPlayGamesStatefulWidget extends StatefulWidget {
  FlyyPlayGame flyyPlayGame;

  FlyyPlayGamesStatefulWidget({Key key, @required this.flyyPlayGame})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyPlayGamesState();
  }
}

class FlyyPlayGamesState extends State<FlyyPlayGamesStatefulWidget> {
  FlyyPlayGame flyyPlayGame;
  final Completer<WebViewController> _controller =
      Completer<WebViewController>();
  bool isLoading;
  int secondsRemaining;

  @override
  void initState() {
    super.initState();
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
    ]);

    isLoading = true;
    flyyPlayGame = widget.flyyPlayGame;
    secondsRemaining = flyyPlayGame.secRemaining;

    if (secondsRemaining == -100 || secondsRemaining > 10) {
      if (secondsRemaining != -100) {
        startTimer((secondsRemaining - 5) * 1000);
      }
    } else {
      FlyyUtility.showToast(
          context, "Tournament Has Ended!!", ToastGravity.BOTTOM);
      SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
      SystemChrome.setEnabledSystemUIOverlays(SystemUiOverlay.values);
      Navigator.pop(context);
    }

    SystemChrome.setEnabledSystemUIOverlays([]);
//    if (flyyPlayGame.isLandscape != null && flyyPlayGame.isLandscape) {
////      SystemChrome.setPreferredOrientations(
////          [DeviceOrientation.landscapeLeft/*, DeviceOrientation.landscapeRight*/]);
//    } else {
//      SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
//    }
  }

  @override
  dispose() {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.landscapeRight,
      DeviceOrientation.landscapeLeft,
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
    ]);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (flyyPlayGame.fullURLToOpen != null) {
      /* SystemChrome.setEnabledSystemUIOverlays([]);
      if (flyyPlayGame.isLandscape != null && flyyPlayGame.isLandscape) {
        SystemChrome.setPreferredOrientations([
          DeviceOrientation.landscapeLeft,
          DeviceOrientation.landscapeRight
        ]);
      } else {
        SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
      }*/
      return WillPopScope(
        onWillPop: onBack,
        child: Scaffold(
          body: Stack(
            alignment: Alignment.center,
            children: <Widget>[
              WebView(
                initialUrl: flyyPlayGame.fullURLToOpen,
                javascriptMode: JavascriptMode.unrestricted,
                debuggingEnabled: true,
                gestureNavigationEnabled: true,
                onPageStarted: (url) {},
                onPageFinished: (url) {
                  setState(() {
                    isLoading = false;
                  });
                },
                onWebResourceError: (error) {
                  Text(error.description,
                      style: TextStyle(fontSize: 20, color: Colors.black));
                },
                navigationDelegate: (request) {
                  if (request.url.startsWith("http"))
                    return NavigationDecision
                        .navigate; //open web links as usual

                  if (request.url.startsWith("market://") ||
                      request.url
                          .contains("play.google.com/store/apps/details")) {
                    try {
                      if (Platform.isAndroid) {
                        final AndroidIntent intent = AndroidIntent(
                            action: 'action_view',
                            data: Uri.encodeFull(request.url));
                        intent.launch();
                      }
                    } catch (e) {
                      FlyyUtility.showToast(
                          context,
                          "No App found to handle this request",
                          ToastGravity.BOTTOM);
                    }
                    return NavigationDecision.prevent;
                  }

                  if (request.url.startsWith("geo")) {
                    try {
                      if (Platform.isAndroid) {
                        final AndroidIntent intent = AndroidIntent(
                            action: 'action_view',
                            package: 'com.google.android.apps.maps',
                            data: 'package:com.google.android.apps.maps');
                        intent.launch();
                      }
                    } catch (e) {
                      FlyyUtility.showToast(
                          context,
                          "No App found to handle this request",
                          ToastGravity.BOTTOM);
                    }
                    return NavigationDecision.prevent;
                  }

                  if (request.url.startsWith("intent://")) {
                    try {
                      if (Platform.isAndroid) {
                        final AndroidIntent intent = AndroidIntent(
                            action: 'action_view',
                            data: Uri.encodeFull(request.url));
                        intent.launch();
                      }
                    } catch (e) {
                      FlyyUtility.showToast(
                          context,
                          "No App found to handle this request",
                          ToastGravity.BOTTOM);
                    }
                    return NavigationDecision.prevent;
                  }

                  if (request.url.startsWith("intent://") ||
                      request.url.startsWith("market://")) {
                    try {
                      if (Platform.isAndroid) {
                        final AndroidIntent intent = AndroidIntent(
                            action: 'action_view',
                            package: 'com.google.android.apps.maps',
                            data: 'package:com.google.android.apps.maps');
                        intent.launch();
                      }
                    } catch (e) {
                      FlyyUtility.showToast(
                          context,
                          "No App found to handle this request",
                          ToastGravity.BOTTOM);
                    }
                    return NavigationDecision.prevent;
                  }

                  if (request.url.contains("android://") &&
                      !request.url.startsWith("android://"))
                    return NavigationDecision.prevent;

                  return NavigationDecision.navigate;
                },
                onWebViewCreated: (WebViewController webViewController) {
                  _controller.complete(webViewController);
                },
              ),
              isLoading
                  ? Center(child: CircularProgressIndicator())
                  : SizedBox.shrink(),
            ],
          ),
        ),
      );
    } else {
      FlyyUtility.showToast(context, "Nothing to Open", ToastGravity.BOTTOM);
      SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
      SystemChrome.setEnabledSystemUIOverlays(SystemUiOverlay.values);
      Navigator.pop(context);
    }
  }

  DateTime currentBackPressTime;

  Future<bool> onBack() async {
    WebViewController webViewController = await _controller.future;
    bool canNavigate = await webViewController.canGoBack();
    print(canNavigate);
    if (canNavigate) {
      webViewController.goBack();
      return false;
    } else {
      DateTime now = DateTime.now();
      if (currentBackPressTime == null ||
          now.difference(currentBackPressTime) > Duration(seconds: 2)) {
        currentBackPressTime = now;
        FlyyUtility.showToast(
            context, "Press back again to Exit!", ToastGravity.BOTTOM);
        return Future.value(false);
      } else {
        SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
        SystemChrome.setEnabledSystemUIOverlays(SystemUiOverlay.values);
        return Future.value(true);
      }
    }
  }

  void startTimer(int endTime) {
    CountDown cd = CountDown(Duration(milliseconds: endTime));
    var sub = cd.stream.listen(null);
    sub.onData((duration) {});

    sub.onDone(() {
      print("Done");
      sub.cancel();
      setState(() {
        FlyyUtility.showToast(
            context, "Tournament Has Ended!!", ToastGravity.BOTTOM);
        SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
        SystemChrome.setEnabledSystemUIOverlays(SystemUiOverlay.values);
        Navigator.pop(context);
      });
    });
  }
}
