import 'dart:async';
import 'dart:convert';

import 'package:app_settings/app_settings.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flyyfluttersdk/Flyy.dart';
import 'package:flyyfluttersdk/helpers/APIClient.dart';
import 'package:flyyfluttersdk/helpers/FlyyCommonColors.dart';
import 'package:flyyfluttersdk/helpers/FlyyGetTimeAgo.dart';
import 'package:flyyfluttersdk/helpers/FlyyUtility.dart';
import 'package:flyyfluttersdk/helpers/ReusableAppbarWidget.dart';
import 'package:flyyfluttersdk/model/FlyyAllTransactionsResponse.dart';
import 'package:flyyfluttersdk/model/FlyyTransactions.dart';
import 'package:flyyfluttersdk/model/FlyyWalletCompleteData.dart';
import 'package:flyyfluttersdk/model/FlyyWalletModel.dart';
import 'package:flyyfluttersdk/views/FlyyGiftCardsPage.dart';
import 'package:flyyfluttersdk/views/FlyyRedeemPage.dart';
import 'package:flyyfluttersdk/views/FlyyTransactionOrderCardDialog.dart';

//class FlyyTransactionsPage extends StatelessWidget {
//  FlyyWalletCompleteData flyyWalletCompleteData;
//  FlyyWalletModel flyyWalletModel;
//
//  FlyyTransactionsPage(
//      {Key key, @required this.flyyWalletCompleteData, this.flyyWalletModel})
//      : super(key: key);
//
//  @override
//  Widget build(BuildContext context) {
//    return Scaffold(
//      appBar: ReusableAppbarWidget.getAppBar(context, 'Transactions'),
//      body: FlyyTransactionsStateFulWidget(
//        flyyWalletCompleteData: flyyWalletCompleteData,
//        flyyWalletModel: flyyWalletModel,
//      ),
//    );
//  }
//}

class FlyyTransactionsPage extends StatefulWidget {
  FlyyWalletCompleteData flyyWalletCompleteData;
  FlyyWalletModel flyyWalletModel;
  bool showLastBoughtGiftCardPopup = false;

  FlyyTransactionsPage(
      {Key key,
      @required this.flyyWalletCompleteData,
      this.flyyWalletModel,
      this.showLastBoughtGiftCardPopup})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return new FlyyTransactionsState();
  }
}

class FlyyTransactionsState extends State<FlyyTransactionsPage> {
  APIClient apiClient;
  FlyyUtility flyyUtility;
  int page = 1;
  bool isLoading = false;
  List<FlyyTransactions> transactionsList;
  FlyyAllTransactionsResponse flyyAllTransactionsResponse;
  bool showLastBoughtGiftCardPopup;

  void displayRecentTransactionDialog() {
    if (transactionsList[0] != null) {
      transactionsList[0].giftCardData != null
          ? showDialog(
              context: context,
              builder: (BuildContext context) => FlyyTransactionOrderCardDialog(
                    flyyGiftCardData: transactionsList[0].giftCardData,
                  ))
          // ignore: unnecessary_statements
          : null;
    }
    showLastBoughtGiftCardPopup = false;
  }

  @override
  Widget build(BuildContext context) {
    apiClient = new APIClient();
    flyyUtility = new FlyyUtility();
    showLastBoughtGiftCardPopup = widget.showLastBoughtGiftCardPopup;
    return Scaffold(
      appBar: ReusableAppbarWidget.getAppBar(context, 'Transactions'),
      body: makeNetworkCall(context),
    );
  }

  ///fetch data
  Widget makeNetworkCall(BuildContext context) {
    return FutureBuilder<bool>(
      future: FlyyUtility.checkIfInterentPresent(),
      builder: (BuildContext context, AsyncSnapshot response) {
        if (response != null && response.data != null && response.data) {
          // Internet Present Case
          return FutureBuilder<FlyyAllTransactionsResponse>(
            future: fetchTransactions(
                page, widget.flyyWalletCompleteData.rewardType),
            builder: (BuildContext context, AsyncSnapshot response) {
              if (response.connectionState == ConnectionState.waiting) {
                return Center(child: CircularProgressIndicator());
              } else {
                if (response.hasData && response.data != null) {
                  if (flyyAllTransactionsResponse != null &&
                      flyyAllTransactionsResponse.flyyTransactions != null) {
                    transactionsList = new List();
                    transactionsList
                        .addAll(flyyAllTransactionsResponse.flyyTransactions);
                  }
                  return setUI();
                } else if (response.hasError) {
                  print(response.error);
//                  noInternetLayout(context);
                }
              }
              return Center(child: CircularProgressIndicator());
            },
          );
        } else {
          // No-Internet Case
          return noInternetLayout(context);
        }
      },
    );
  }

  Widget noInternetLayout(BuildContext context) {
    return Container(
      margin: const EdgeInsets.all(20),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Image.network(
            "https://ghc.theflyy.com/assets/fs_assets/ic_no_internet.png",
            height: 80,
            width: 80,
          ),
          Padding(
            padding: const EdgeInsets.only(top: 10, bottom: 10),
            child: Text(
              "Couldn't connect to the internet. Please check your network.",
              textAlign: TextAlign.center,
              style: TextStyle(
                  fontSize: 15, color: FlyyUtility.parseColor('#575555')),
            ),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: <Widget>[
              GestureDetector(
                onTap: () {
                  makeNetworkCall(context);
                },
                child: Row(
                  children: <Widget>[
                    Icon(
                      Icons.refresh,
                      size: 30,
                      color: FlyyUtility.parseColor('#717171'),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Tap To Retry',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
              GestureDetector(
                onTap: () {
                  AppSettings.openWIFISettings();
                },
                child: Row(
                  children: <Widget>[
                    Image.network(
                      "https://ghc.theflyy.com/assets/fs_assets/ic_settings.png",
                      height: 30,
                      width: 30,
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 5),
                      child: Text(
                        'Go To Settings',
                        style: TextStyle(
                            fontSize: 14,
                            color: FlyyUtility.parseColor('#717171')),
                      ),
                    )
                  ],
                ),
              ),
            ],
          )
        ],
      ),
    );
  }

  Widget setUI() {
    if (showLastBoughtGiftCardPopup) {
      Timer(Duration(seconds: 1), () {
        displayRecentTransactionDialog();
      });
    }
    return Container(
      height: double.maxFinite,
      color: FlyyCommonColors.BG_GREY,
      child: SingleChildScrollView(
        child: Container(
          margin: const EdgeInsets.all(5),
          child: Card(
            child: Column(
              children: <Widget>[
                ///balance and redeem layout
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    ///reward icon and balance layout
                    Padding(
                      padding: const EdgeInsets.only(
                          left: 8.0, right: 8, top: 8, bottom: 15),
                      child: Row(
                        children: <Widget>[
                          ///reward icon
                          widget.flyyWalletCompleteData != null &&
                                  widget.flyyWalletCompleteData.rewardIcon !=
                                      null &&
                                  widget.flyyWalletCompleteData.rewardIcon
                                      .isNotEmpty
                              ? Image.network(
                                  widget.flyyWalletCompleteData.rewardIcon,
                                  height: 35,
                                  width: 35,
                                )
                              : SizedBox.shrink(),

                          ///balances layout
                          Padding(
                            padding: const EdgeInsets.only(left: 5),
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                ///balance static text
                                Text(
                                  'Balance',
                                  style: TextStyle(
                                      fontSize: 12,
                                      color: FlyyUtility.parseColor('#736c6c')),
                                ),

                                ///balance text dynamic
                                Text(
                                  flyyUtility.setRewardText(
                                      widget.flyyWalletCompleteData.balance,
                                      widget.flyyWalletCompleteData.rewardType),
                                  style: TextStyle(
                                      fontSize: 16,
                                      color: FlyyUtility.parseColor('#736c6c')),
                                ),
                              ],
                            ),
                          ),
                        ],
                      ),
                    ),

                    ///Redeem button
                    FlyyUtility.equalsIgnoreCase(
                                widget.flyyWalletCompleteData.rewardType,
                                'Cash') ||
                            widget.flyyWalletModel.activeCurrency != null &&
                                widget.flyyWalletModel.activeCurrency.label ==
                                    widget.flyyWalletCompleteData.rewardType
                        ? GestureDetector(
                            onTap: () {
                              if (widget.flyyWalletCompleteData.rewardType !=
                                      null &&
                                  FlyyUtility.equalsIgnoreCase(
                                      widget.flyyWalletCompleteData.rewardType,
                                      'Cash')) {
                                if (widget.flyyWalletCompleteData.balance >=
                                    widget.flyyWalletCompleteData
                                        .minRedeemAmount) {
                                  Navigator.push(
                                    context,
                                    MaterialPageRoute(
                                        builder: (context) => FlyyRedeemPage(
                                              flyyWalletCompleteData:
                                                  widget.flyyWalletCompleteData,
                                            )),
                                  );
                                }
                              } else {
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                      builder: (context) => FlyyGiftCardsPage(
                                            flyyWalletCompleteData:
                                                widget.flyyWalletCompleteData,
                                            flyyWalletModel:
                                                widget.flyyWalletModel,
                                          )),
                                );
                              }
                            },
                            child: setRedeemIconAsPerBalance(
                                widget.flyyWalletCompleteData,
                                widget.flyyWalletModel),
                          )
                        : SizedBox.shrink()
                  ],
                ),

                ///Divider

                ///listview to display all transactions
                transactionsList != null && transactionsList.length > 0
                    ? Column(
                        children: <Widget>[
                          Container(
                            width: double.infinity,
                            height: 1,
                            margin: const EdgeInsets.only(
                                left: 20, right: 20, bottom: 5, top: 5),
                            child: Divider(
                              color: FlyyUtility.parseColor("#D3D3D3"),
                            ),
                          ),
                          Center(
                            child: Container(
                              margin: const EdgeInsets.all(10),
                              child: Column(
                                children: <Widget>[
                                  NotificationListener<ScrollNotification>(
                                    onNotification:
                                        (ScrollNotification scrollInfo) {
                                      if (!isLoading &&
                                          scrollInfo.metrics.pixels ==
                                              scrollInfo
                                                  .metrics.maxScrollExtent) {
                                        FlyyUtility.checkIfInterentPresent()
                                            .then((isConnected) {
                                          if (isConnected != null &&
                                              isConnected) {
                                            // Internet Present Case
                                            fetchTransactions(
                                                page++,
                                                widget.flyyWalletCompleteData
                                                    .rewardType);
                                            // start loading data
                                            isLoading = true;
                                          } else {
                                            // No-Internet Case
                                            isLoading = false;
                                            FlyyUtility.showToast(
                                                context,
                                                ("Please check your internet connection and try again."),
                                                ToastGravity.BOTTOM);
                                          }
                                        });
                                      }
                                      return true;
                                    },
                                    child: ListView.separated(
                                      separatorBuilder: (context, index) =>
                                          Divider(
                                        color:
                                            FlyyUtility.parseColor("#D3D3D3"),
                                      ),
                                      itemBuilder: setTransactionsListView,
                                      itemCount: transactionsList.length,
                                      scrollDirection: Axis.vertical,
                                      shrinkWrap: true,
                                      physics:
                                          const NeverScrollableScrollPhysics(),
                                    ),
                                  ),
                                  Container(
                                    height: isLoading ? 50.0 : 0,
                                    color: Colors.transparent,
                                    child: Center(
                                      child: new CircularProgressIndicator(),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          ),
                        ],
                      )
                    : SizedBox.shrink(),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget setRedeemIconAsPerBalance(
      FlyyWalletCompleteData flyyWalletCompleteData,
      FlyyWalletModel flyyWalletModel) {
    if ((flyyWalletModel.activeCurrency != null &&
        flyyWalletModel.activeCurrency.label ==
            flyyWalletCompleteData.rewardType)) {
      return commonRedeemUi(Flyy.TOOLBAR_COLOR);
    }

    if (flyyWalletCompleteData.rewardType != null &&
        FlyyUtility.equalsIgnoreCase(
            flyyWalletCompleteData.rewardType, 'Cash')) {
      if (flyyWalletCompleteData.balance != null &&
          flyyWalletCompleteData.minRedeemAmount != null &&
          flyyWalletCompleteData.balance >=
              flyyWalletCompleteData.minRedeemAmount) {
        return commonRedeemUi(Flyy.TOOLBAR_COLOR);
      } else {
        return commonRedeemUi(FlyyUtility.parseColor('#aaa'));
      }
    }
  }

  Widget commonRedeemUi(Color containerColor) {
    return Container(
      margin: const EdgeInsets.only(right: 10),
      alignment: Alignment.topRight,
      child: Card(
        color: containerColor,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(20),
        ),
        child: Padding(
          padding: const EdgeInsets.all(10),
          child: Row(
            children: <Widget>[
              SvgPicture.network(
                "https://ghc.theflyy.com/assets/fs_assets/ic_redeem.svg",
                height: 15,
                width: 15,
              ),
              Padding(
                padding: const EdgeInsets.only(left: 5),
                child: Text(
                  'Redeem',
                  style: TextStyle(fontSize: 14, color: Colors.white),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }

  FlyyGetTimeAgo flyyGetTimeAgo = new FlyyGetTimeAgo();

  Widget setTransactionsListView(BuildContext context, int index) {
    FlyyTransactions flyyTransactions = transactionsList[index];
    return GestureDetector(
      onTap: () {
        flyyTransactions.giftCardData != null
            ? showDialog(
                context: context,
                builder: (BuildContext context) =>
                    FlyyTransactionOrderCardDialog(
                      flyyGiftCardData: flyyTransactions.giftCardData,
                    ))
            : null;
      },
      child: Container(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            ///title and description
            Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Text(
                  flyyTransactions.message != null
                      ? flyyTransactions.message
                      : "",
                  style: TextStyle(
                      fontSize: 12, color: FlyyUtility.parseColor('#544e4e')),
                ),
                Text(
                  getTransactionReferenceText(flyyTransactions),
                  style: TextStyle(
                      fontSize: 12, color: FlyyUtility.parseColor('#544e4e')),
                )
              ],
            ),

            ///cash/coins and created at
            Column(
              crossAxisAlignment: CrossAxisAlignment.end,
              children: <Widget>[
                setTextAsPerEarned(context, flyyTransactions),
                Text(
                  flyyGetTimeAgo.timeAgo(flyyTransactions.createdAt),
                  style: TextStyle(
                      fontSize: 12, color: FlyyUtility.parseColor('#544e4e')),
                )
              ],
            )
          ],
        ),
      ),
    );
  }

  Widget setTextAsPerEarned(
      BuildContext context, FlyyTransactions flyyTransactions) {
    if (flyyTransactions.isEarned == ('true')) {
      return Text(
        '+ ' +
            flyyUtility.setRewardText(
                flyyTransactions.value, flyyTransactions.prizeType),
        style:
            TextStyle(fontSize: 12, color: FlyyUtility.parseColor('#2BA037')),
      );
    } else {
      return Text(
        '- ' +
            flyyUtility.setRewardText(
                flyyTransactions.value, flyyTransactions.prizeType),
        style:
            TextStyle(fontSize: 12, color: FlyyUtility.parseColor('#d0021b')),
      );
    }
  }

  String getTransactionReferenceText(FlyyTransactions flyyTransactions) {
    if (flyyTransactions.reference != null &&
        flyyTransactions.reference.isNotEmpty)
      return flyyTransactions.reference;
    else if (flyyTransactions.creditReference != null &&
        flyyTransactions.creditReference.isNotEmpty)
      return flyyTransactions.creditReference;
    else if (flyyTransactions.debitReference != null &&
        flyyTransactions.debitReference.isNotEmpty)
      return flyyTransactions.debitReference;
  }

  Future<FlyyAllTransactionsResponse> fetchTransactions(
      int page, String rewardType) async {
    apiClient = new APIClient();
    final response = await apiClient
        .getClient("web_cash_transactions?page=$page&prize_type=$rewardType");
    if (response.statusCode == 200) {
      final responseJson = json.decode(response.body);
      flyyAllTransactionsResponse =
          FlyyAllTransactionsResponse.fromJson(responseJson);

      if (transactionsList == null) transactionsList = new List();
      if (flyyAllTransactionsResponse.flyyTransactions != null &&
          flyyAllTransactionsResponse.flyyTransactions.length > 0)
        transactionsList.addAll(flyyAllTransactionsResponse.flyyTransactions);
      isLoading = false;

      return flyyAllTransactionsResponse;
    } else {
      throw Exception('Failed to load data for reward ' + response.toString());
    }
  }
}
